import React, { useEffect } from 'react';
import ColorPicker from '@graphpolaris/shared/lib/components/colorComponents/colorPicker';
import { MapProps } from '../../mapvis';
import { Button, DropdownColorLegend, EntityPill, Input, RelationPill } from '@graphpolaris/shared/lib/components';
import { LayerSettingsComponentType } from '../../mapvis.types';

const areaColoringStrategies = ['Node count', 'Edge count', 'Incoming edges', 'Outgoing edges', 'Connected edges', 'Attribute'];

export type coloringStrategiesType = 'Node count' | 'Edge count' | 'Incoming edges' | 'Outgoing edges' | 'Connected edges' | 'Attribute';

export function ChoroplethOptions({
  settings,
  graphMetadata,
  spatialAttributes,
  updateLayerSettings,
  updateSpatialAttribute,
}: LayerSettingsComponentType<MapProps>) {
  const layerType = 'choropleth';
  const layerSettings = settings[layerType];

  useEffect(() => {
    if (!layerSettings) {
      const initialSettingsObject = { coloringStrategy: 'Node count', colorScale: 'orange', opacity: 0.8, nodes: {}, edges: {} };

      graphMetadata.nodes.labels.forEach((node) => {
        initialSettingsObject.nodes = {
          ...initialSettingsObject.nodes,
          [node]: {
            color: [0, 0, 0],
            hidden: false,
            fixed: true,
            min: 0,
            max: 10,
            sizeAttribute: '',
          },
        };
      });

      graphMetadata.edges.labels.forEach((edge) => {
        initialSettingsObject.edges = {
          ...initialSettingsObject.edges,
          [edge]: {
            color: [0, 0, 0],
            onHover: true,
          },
        };
      });

      updateLayerSettings({ ...initialSettingsObject });
    }
  }, [graphMetadata, layerType, settings, updateLayerSettings]);

  const handleCollapseToggle = (type: string, itemType: 'nodes' | 'edges') => {
    if (layerSettings) {
      updateLayerSettings({
        [itemType]: {
          ...layerSettings[itemType],
          [type]: {
            ...layerSettings[itemType][type],
            collapsed: !layerSettings[itemType][type]?.collapsed ?? true,
          },
        },
      });
    }
  };

  return (
    layerSettings && (
      <div>
        <div className="mt-2">
          <p className="text-bold">Area color</p>
          <Input
            inline
            label="Based on"
            type="dropdown"
            value={layerSettings?.coloringStrategy}
            options={areaColoringStrategies}
            onChange={(val) => updateLayerSettings({ coloringStrategy: val as coloringStrategiesType })}
          />
          <DropdownColorLegend value={settings?.colorScale} onChange={(val) => updateLayerSettings({ colorScale: val })} />
          <Input
            label="Opacity"
            type="slider"
            min={0}
            max={1}
            step={0.05}
            unit="%"
            value={layerSettings?.opacity ?? 0.8}
            onChange={(val) => updateLayerSettings({ opacity: val as number })}
          />
        </div>

        {graphMetadata.nodes.labels.map((nodeType) => {
          const nodeSettings = layerSettings?.nodes?.[nodeType] || {};

          return (
            <div className="mt-2" key={nodeType}>
              <div className="flex items-center">
                <Button
                  size="2xs"
                  iconComponent={nodeSettings.collapsed ? 'icon-[ic--baseline-arrow-right]' : 'icon-[ic--baseline-arrow-drop-down]'}
                  variant="ghost"
                  onClick={() => handleCollapseToggle(nodeType, 'nodes')}
                />
                <div className="w-3/4 mr-6 cursor-pointer" onClick={() => handleCollapseToggle(nodeType, 'nodes')}>
                  <EntityPill title={nodeType} />
                </div>
              </div>

              {!nodeSettings.collapsed && (
                <div>
                  <Input
                    inline
                    label="Latitude"
                    type="dropdown"
                    value={settings?.location[nodeType].lat}
                    options={[...spatialAttributes[nodeType]]}
                    disabled={spatialAttributes[nodeType].length < 1}
                    onChange={(val) => updateSpatialAttribute(nodeType, 'lat', val as string)}
                  />
                  <Input
                    inline
                    label="Longitude"
                    type="dropdown"
                    value={settings?.location[nodeType].lon}
                    options={[...spatialAttributes[nodeType]]}
                    disabled={spatialAttributes[nodeType].length < 1}
                    onChange={(val) => updateSpatialAttribute(nodeType, 'lon', val as string)}
                  />
                </div>
              )}
            </div>
          );
        })}

        {graphMetadata.edges.labels.map((edgeType) => {
          const edgeSettings = layerSettings?.edges?.[edgeType] || {};

          return (
            <div key={edgeType} className="mt-2">
              <div className="flex items-center">
                <Button
                  size="2xs"
                  iconComponent={edgeSettings?.collapsed ? 'icon-[ic--baseline-arrow-right]' : 'icon-[ic--baseline-arrow-drop-down]'}
                  variant="ghost"
                  onClick={() => handleCollapseToggle(edgeType, 'edges')}
                />
                <div className="w-3/4 mr-6 cursor-pointer" onClick={() => handleCollapseToggle(edgeType, 'edges')}>
                  <RelationPill title={edgeType} />
                </div>
              </div>

              {!edgeSettings.collapsed && (
                <div>
                  <div className="flex justify-between">
                    <span className="font-bold">Color</span>
                    <ColorPicker
                      value={edgeSettings.color}
                      updateValue={(val) =>
                        updateLayerSettings({ edges: { ...settings.edges, [edgeType]: { ...edgeSettings, color: val } } })
                      }
                    />
                  </div>

                  <Input
                    label="Edges on hover"
                    type="boolean"
                    value={layerSettings?.enableBrushing}
                    onChange={(val) => {
                      updateLayerSettings({ enableBrushing: val as boolean });
                    }}
                  />
                </div>
              )}
            </div>
          );
        })}
      </div>
    )
  );
}
